//
//  ViewController.swift
//  Swift-镂空遮罩
//
//  Created by 李含文 on 2019/7/30.
//  Copyright © 2019年 SK丿希望. All rights reserved.
//

import UIKit

extension UIView {
    
    /// 快速添加镂空View
    ///
    /// - Parameters:
    ///   - radius: 圆角半径(nil:默认半径)
    ///   - color: 背景颜色(nil:看自己又没有就自己 没有就去取父控件的)
    ///   - corners: 切除部分(nil:默认切4角)
    func addHollowOutView(_ radius: CGFloat? = nil, _ color: UIColor? = nil, _ corners: UIRectCorner? = nil) {
        let rect = bounds
        let backgroundView = UIView(frame: rect) // 创建背景View
        backgroundView.isUserInteractionEnabled = false // 不接收事件 不然会阻挡原始事件触发
        var currentcolor = color ?? backgroundColor // 设置颜色
        if currentcolor == nil { // 如果没设置背景色
            if let superView = self.superview { // 看看父控件是否存在 存在直接用父控件背景色
                currentcolor = superView.backgroundColor
            } else { // 不然给定白色
                currentcolor = UIColor.white
            }
        }
        backgroundView.backgroundColor = currentcolor
        let currentradius:CGFloat = radius ?? rect.size.height*0.5 // 设置圆角半径
        self.addSubview(backgroundView) // 添加遮罩层
        self.bringSubviewToFront(backgroundView) // 放置到最顶层

        let maskLayer = CAShapeLayer()
        maskLayer.fillRule = CAShapeLayerFillRule.evenOdd //  奇偶层显示规则
        let basicPath = UIBezierPath(rect: rect) // 底色
        
        let radii = CGSize(width: currentradius, height: currentradius)
        let currentcorners = corners ?? [UIRectCorner.topRight,UIRectCorner.bottomRight,UIRectCorner.bottomLeft,UIRectCorner.topLeft]
        let maskPath = UIBezierPath(roundedRect: rect, byRoundingCorners: currentcorners, cornerRadii: radii) // 镂空路径
//        let maskPath = UIBezierPath(ovalIn: frame)
        basicPath.append(maskPath) // 重叠
        maskLayer.path = basicPath.cgPath
        backgroundView.layer.mask = maskLayer
    }
}

class ViewController: UIViewController {

    private lazy var imageView: UIImageView = {
        let imageView = UIImageView()
//        imageView.backgroundColor = UIColor.white
        imageView.contentMode = .scaleAspectFill
        imageView.clipsToBounds = true
        let frame = CGRect(x: 0, y: 0, width: 100, height: 100)
        imageView.frame = frame
        let image = UIImage(named: "图片")
        imageView.image = image
        imageView.isUserInteractionEnabled = true
        let tap = UITapGestureRecognizer.init(target: self, action: #selector(viewGestureTrigger))
        tap.numberOfTouchesRequired = 1 //点击出点需要几个点
        tap.numberOfTapsRequired = 1 //连续点击几次才能引发手势事件
        imageView.addGestureRecognizer(tap)
        return imageView
    }()
    
    var containerView: UIView!
    override func viewDidLoad() {
        super.viewDidLoad()
        view.addSubview(imageView)
//        imageView.addHollowOutView(10)
//        imageView.addHollowOutView()
        imageView.addHollowOutView(nil, nil, [UIRectCorner.topRight,UIRectCorner.bottomRight,UIRectCorner.bottomLeft])
//        imageView.addHollowOutView(nil,UIColor.red)
        imageView.center = self.view.center
        
//        self.containerView = UIView()
//        self.containerView.frame = CGRect(x: 0, y: 100, width: 375, height: 375)
//        self.containerView.backgroundColor = UIColor.groupTableViewBackground
//        self.view.addSubview(self.containerView)
//        let rect = CGRect(x: 50, y: 100, width: 100, height: 100)
//        let radii = CGSize(width: 20, height: 20)
//        let corners: UIRectCorner = [UIRectCorner.topRight,UIRectCorner.bottomRight,UIRectCorner.bottomLeft]
//        let path = UIBezierPath(roundedRect: rect, byRoundingCorners: corners, cornerRadii: radii)
//        let shapeLayer = CAShapeLayer()
//        // 设置路径的颜色
//        shapeLayer.strokeColor = UIColor.red.cgColor
//        // 路径的填充
//        shapeLayer.fillColor = UIColor.clear.cgColor
//        // 设置路径的宽度
//        shapeLayer.lineWidth = 1
//        // 设置路径结合处的形状  'miter'(切角), `round'（圆弧） and `bevel'（正切角）.
//        shapeLayer.lineJoin = CAShapeLayerLineJoin.round
//        // 设置路径的尾端形状 'butt'(和 square 一样，矩形), `round'（半圆形）* and `square'（矩形）。
//        shapeLayer.lineCap = CAShapeLayerLineCap.round
//         // 添加路径
//        shapeLayer.path = path.cgPath
//        self.containerView.layer.addSublayer(shapeLayer)
    }

    /// 点击回调
    @objc private func viewGestureTrigger(){
        print("点击了图片")
    }
}

